<?php

require_once __DIR__ . '/Tables.php';

class Database{
    private $host = DB_HOST;
    private $user = DB_USER;
    private $password = DB_PASSWORD;
    private $port = DB_PORT;
    private $name = DB_NAME;
    private $db_connection;
    private $statement;

    public function __construct(){
        $dsn = "pgsql:host=" . $this->host . ";dbname=" . $this->name . ";user=" . $this->user . ";password=" . $this->password . ";port=" . $this->port;
        
        try{
            $this->db_connection = new PDO($dsn);
        } catch (PDOException $e){
            echo($e->getMessage());
        }

        try{
            $this->db_connection->exec((Tables::USER_TABLE));
            $this->db_connection->exec((Tables::NOTE_TABLE));
            $this->db_connection->exec((Tables::REMINDER_TABLE));
            $this->db_connection->exec((Tables::TAG_TABLE));
            $this->db_connection->exec((Tables::TAG_NOTE_TABLE));
            $this->db_connection->exec((Tables::CATEGORY_REMINDER_TABLE));
            
        } catch(PDOException $e){
            echo($e->getMessage());
        }
    }

    public function query($query){
        try{
            $this->statement = $this->db_connection->prepare(($query));
        } catch (PDOException $e){
            echo($e->getMessage());
        }
    }

    public function bind($param, $value, $type = null){
        try{
            if(is_null($type)){
                if(is_int($value)){
                    $type = PDO::PARAM_INT;
                } else if (is_bool($value)){
                    $type = PDO::PARAM_BOOL;
                } else if (is_bool($value)){
                    $type = PDO::PARAM_NULL;
                } else {
                    $type = PDO::PARAM_STR;
                }
            }
            $this->statement->bindValue($param, $value, $type);
        } catch(PDOException $e){
            echo($e->getMessage());
        }
    }

    public function execute(){
        try{
            $this->statement->execute();
        } catch(PDOException $e){
            echo($e->getMessage());
        }
    }

    public function fetch(){
        try{
            $this->execute();
            return $this->statement->fetch(PDO::FETCH_OBJ);
        } catch(PDOException $e){
            echo($e->getMessage());
        }
    }

    public function fetchAll(){
        try{
            $this->execute();
            return $this->statement->fetchAll(PDO::FETCH_OBJ);
        } catch(PDOException $e){
            echo($e->getMessage());
        }
    }

    public function rowCount(){
        try{
            return $this->statement->rowCount();
        } catch(PDOException $e){
            echo($e->getMessage());
        }
    }

    public function lastInsertId() {
        try{
            return $this->db_connection->lastInsertId();
        } catch(PDOException $e){
            echo($e->getMessage());
        }
    }
}
