<?php
require_once __DIR__ .'/../../config/config.php';
require_once __DIR__ .'/../../app/core/Database.php';
require_once __DIR__ . '/CategoryRemindersModel.php';
require_once __DIR__ .'/../../app/core/DataSeeder.php';
class ReminderModel
{
  private $table = 'reminders';
  private $db;
  private $CategoryRemindersModel;

  public function __construct()
  {
    
    $this->db = new Database;
    $this->CategoryRemindersModel = new CategoryRemindersModel;
    
  }

  public function getAllReminders()
  {
    $this->db->query('SELECT * FROM ' . $this->table);
    return $this->db->fetchAll();
  }

  public function getReminderById($id)
  {
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE reminder_id = :id');
    $this->db->bind('id', $id);
    return $this->db->fetch();
  }
  public function getReminderByUserId($user_id)
  {
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE user_id = :id');
    $this->db->bind('id', $user_id);

    return $this->db->fetchAll();
  }
  public function insertReminder($data)
  {
    $this->db->query('INSERT INTO ' . $this->table . ' (user_id, title, deadline, tasks) VALUES (:user_id, :title, :deadline, :tasks)');
    
    $this->db->bind('user_id', $data['user_id']);
    $this->db->bind('title', $data['title']);
    $this->db->bind('deadline', $data['deadline']);
    $this->db->bind('tasks', $data['tasks']);

    $this->db->execute();

    $lastInsertedId = $this->db->lastInsertId();

    $data['reminder_id'] = $lastInsertedId;

    if (isset($data['category']) && !empty($data['category']) && isset($data['reminder_id']) && !empty($data['reminder_id'])) {
      $this->CategoryRemindersModel->insertReminderCategory($data);
    }

    return $this->db->rowCount();
  }

  public function updateReminder($data)
  {
    $this->db->query('UPDATE ' . $this->table . ' SET user_id = :user_id, title = :title, deadline = :deadline, tasks = :tasks WHERE reminder_id = :reminder_id');

    $this->db->bind('user_id', $data['user_id']);
    $this->db->bind('title', $data['title']);
    $this->db->bind('deadline', $data['deadline']);
    $this->db->bind('tasks', $data['tasks']);
    $this->db->bind('reminder_id', $data['reminder_id']);

    $this->db->execute();
    if (isset($data['category']) && !empty($data['category'])) {
      $this->CategoryRemindersModel->updateReminderCategory($data);
    }

    return $this->db->rowCount();
  }

  public function deleteReminder($data)
  {
    if (isset($data['category']) && !empty($data['category'])) {
      $this->CategoryRemindersModel->deleteReminderCategory($data);
    }
    
    $this->db->query('DELETE FROM ' . $this->table . ' WHERE reminder_id = :reminder_id');

    $this->db->bind('reminder_id', $data['reminder_id']);

    $this->db->execute();

    return $this->db->rowCount();
  }

  public function getCategoryByReminderId($id)
  {
    $this->db->query('SELECT * FROM category_reminders WHERE reminder_id = :id');
    
    $this->db->bind('id', $id);
    return $this->db->fetch();
  }
  public function findReminders($data)
  {
    $title = '%' . $data['title'] . '%';
    
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE title LIKE :title');
    $this->db->bind('title', $title);

    return $this->db->fetchAll();
  }

  public function filterReminders($data){
    $this->CategoryRemindersModel->getReminderByCategory($data);
  }
}