<?php

require_once __DIR__ .'/../../config/config.php';
require_once __DIR__ .'/../../app/core/Database.php';

class NoteModel
{
  private $table = 'notes';
  private $db;

  public function __construct()
  {
    $this->db = new Database;
  }

  public function getAllNotes()
  {
    $this->db->query('SELECT * FROM ' . $this->table);
    return $this->db->fetchAll();
  }

  // public function getLatestNoteId($user_id = "")
  // {
  //   $this->db->query('SELECT note_id FROM ' . $this->table . ' ORDER BY note_id DESC LIMIT 1');  
  //   return $this->db->fetch();
  // }

  public function getLatestNoteId($user_id = "")
  {
    $query = 'SELECT note_id FROM ' . $this->table;
    if($user_id != "") {
      $query .= ' WHERE user_id = :user_id';
    }

    $query .= ' ORDER BY note_id DESC LIMIT 1';

    $this->db->query($query);
    if($user_id != "") {
      $this->db->bind('user_id', $user_id);
    }
  
    return $this->db->fetch();
  }
  

  // public function getCountNotes()
  // {
  //   $this->db->query('SELECT COUNT(note_id) as num_note FROM tag_notes');

  //   return $this->db->fetch();
  // }
  
  public function getCountNotes($data) {
    $query = "SELECT COUNT(n.note_id) num_note FROM " . $this->table . " n JOIN tag_notes t ON n.note_id = t.note_id";
    $where = false;
    $binds = array();

    if (isset($data['user_id'])) {
      $query .= " WHERE n.user_id = :user_id";
      $binds['user_id'] = $data['user_id'];
    }

    if (isset($data['title'])) {
      $query .= " AND LOWER(n.title) LIKE :title";
      // $query .= " WHERE LOWER(n.title) LIKE :title";
      $binds['title'] = '%' . strtolower($data['title']) . '%';
      $where = true;
    }

    // Memproses tags jika disediakan
    if (isset($data['tags']) && !empty($data['tags'])) {
      if ($where) {
        $query .= " AND t.tag_name IN (";

        // Menggunakan placeholder untuk setiap tag
        $tags = explode(',', $data['tags']);
        foreach ($tags as $index => $tag) {
          $query .= ":tag" . $index . ",";
          $binds['tag' . $index] = trim($tag);
        }

        // Menghapus koma ekstra dan menutup kurung
        $query = rtrim($query, ',') . ")";
      } else {
        // Jika tidak ada kondisi WHERE sebelumnya, gunakan WHERE
        $query .= " WHERE t.tag_name IN (";

        // Menggunakan placeholder untuk setiap tag
        $tags = explode(',', $data['tags']);
        foreach ($tags as $index => $tag) {
          $query .= ":tag" . $index . ",";
          $binds['tag' . $index] = trim($tag);
        }

        // Menghapus koma ekstra dan menutup kurung
        $query = rtrim($query, ',') . ")";
      }
    }

    // Persiapan prepared statement
    $this->db->query($query);

    // Bind parameters
    foreach ($binds as $key => $value) {
        $this->db->bind($key, $value);
    }

    // Eksekusi prepared statement
    $this->db->execute();

    return $this->db->fetch();
  }

  public function getUserByNoteId($noteId)
  {
    $this->db->query('SELECT user_id FROM ' . $this->table . ' WHERE note_id = :id');
    $this->db->bind('id', $noteId);

    return $this->db->fetch();
  }

  public function getNoteById($id)
  {
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE note_id = :id');
    $this->db->bind('id', $id);

    return $this->db->fetch();
  }

  public function getNotesByUserId($id)
  {
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE user_id = :id');
    $this->db->bind('id', $id);

    return $this->db->fetchAll();
  }

  public function getNotesAndTagsByUserId($id)
  {
    $this->db->query('SELECT 1 FROM ' . $this->table . ' n JOIN tag_notes t ON n.note_id = t.note_id WHERE user_id = :id');
    $this->db->bind('id', $id);

    return $this->db->fetchAll();
  }

  public function getNotesByUserIdAndPage($id, $page) {
    $resultsPerPage = 4; // Jumlah hasil per halaman
    $offset = ($page - 1) * $resultsPerPage; // Hitung offset berdasarkan halaman

    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE user_id = :id LIMIT :limit OFFSET :offset');

    $this->db->bind('id', $id);
    $this->db->bind('limit', $resultsPerPage);
    $this->db->bind('offset', $offset);

    $this->db->execute();
    return $this->db->fetchAll();
  }

  public function insertNote($data){
    // Kueri SQL dengan prepared statement
    $this->db->query('INSERT INTO ' . $this->table . ' (title, content, user_id, video_path, image_path) VALUES (:title, :content, :user_id, :video_path, :image_path)');
    
    // Bind parameter dengan nilai yang sesuai
    $this->db->bind('title', $data['title']);
    $this->db->bind('content', $data['content']);
    $this->db->bind('user_id', $data['user_id']);
    $this->db->bind('video_path', $data['video_path']);
    $this->db->bind('image_path', $data['image_path']);

    // Eksekusi prepared statement
    $this->db->execute();

    // Mengembalikan jumlah baris yang terpengaruh (biasanya 1 jika berhasil)
    return $this->db->rowCount();
  }
  
  public function editNote($data) {
    // Kueri SQL dengan prepared statement
    $this->db->query('UPDATE ' . $this->table . ' SET title = :title, content = :content, video_path = :video_path, image_path = :image_path WHERE note_id = :id');

    // Bind parameter dengan nilai yang sesuai
    $this->db->bind('title', $data->title);
    $this->db->bind('content', $data->content);
    $this->db->bind('video_path', $data->video_path);
    $this->db->bind('image_path', $data->image_path);
    $this->db->bind('id', $data->note_id);

    // Eksekusi prepared statement
    $this->db->execute();

    // Mengembalikan jumlah baris yang terpengaruh (biasanya 1 jika berhasil)
    // return $stmt->rowCount();
    return true;
  }

  public function deleteNote($id) {
    $this->db->query('DELETE FROM ' . $this->table . ' WHERE note_id = :id');

    $this->db->bind('id', $id);
    return $this->db->fetch();
  }

  
  // public function findNotes($data) {
  //   $query = "SELECT * FROM " . $this->table . " n JOIN tag_notes t ON n.note_id = t.note_id";
  //   $where = false;
  //   $binds = array();

  //   if (isset($data['user_id'])) {
  //     $query .= " WHERE n.user_id = :user_id";
  //     $binds['user_id'] = $data['user_id'];
  //   }

  //   if (isset($data['title'])) {
  //     if (isset($data['user_id'])) {
  //       $query .= " AND LOWER(n.title) LIKE :title";
  //     } else {
  //       $query .= " WHERE LOWER(n.title) LIKE :title";
  //     }

  //     $binds['title'] = '%' . strtolower($data['title']) . '%';
  //     $where = true;
  //   }

  //   // Memproses tags jika disediakan
  //   if (isset($data['tags']) && !empty($data['tags'])) {
  //     if ($where) {
  //       $query .= " AND t.tag_name IN (";

  //       // Menggunakan placeholder untuk setiap tag
  //       $tags = explode(',', $data['tags']);
  //       foreach ($tags as $index => $tag) {
  //         $query .= ":tag" . $index . ",";
  //         $binds['tag' . $index] = trim($tag);
  //       }

  //       // Menghapus koma ekstra dan menutup kurung
  //       $query = rtrim($query, ',') . ")";
  //     } else {
  //       // Jika tidak ada kondisi WHERE sebelumnya, gunakan WHERE
  //       $query .= " WHERE t.tag_name IN (";

  //       // Menggunakan placeholder untuk setiap tag
  //       $tags = explode(',', $data['tags']);
  //       foreach ($tags as $index => $tag) {
  //         $query .= ":tag" . $index . ",";
  //         $binds['tag' . $index] = trim($tag);
  //       }

  //       // Menghapus koma ekstra dan menutup kurung
  //       $query = rtrim($query, ',') . ")";
  //     }
  //   }

  //   $query .= " ORDER BY n.note_id";
  //   if (isset($data['sort'])) {
  //     if ($data['sort'] == 1) {
  //       $query .= " ASC";
  //     }
  //   } else {
  //     $query .= " DESC";
  //   }

  //   $query .= " LIMIT 4";

  //   // Menambahkan OFFSET jika halaman diberikan
  //   if (isset($data['page'])) {
  //     $query .= " OFFSET :offset";
  //     $binds['offset'] = $data['limit'] * ($data['page'] - 1);
  //   } else {
  //     $query .= " OFFSET 0";
  //   }

  //   // Persiapan prepared statement
  //   $this->db->query($query);

  //   // Bind parameters
  //   foreach ($binds as $key => $value) {
  //       $this->db->bind($key, $value);
  //   }

  //   // Eksekusi prepared statement
  //   $this->db->execute();

  //   return $this->db->fetchAll();
  // }

  public function findNotes($data) {
    $query = "SELECT * FROM " . $this->table . " n JOIN tag_notes t ON n.note_id = t.note_id";
    $where = false;
    $binds = array();
  
    if (isset($data['user_id'])) {
      $query .= " WHERE n.user_id = :user_id";
      $binds['user_id'] = $data['user_id'];
      $where = true;
    }
  
    if (isset($data['title'])) {
      if ($where) {
        $query .= " AND LOWER(n.title) LIKE :title";
      } else {
        $query .= " WHERE LOWER(n.title) LIKE :title";
        $where = true;
      }
      
      $binds['title'] = '%' . strtolower($data['title']) . '%';
    }
  
    // Memproses tags jika disediakan
    if (isset($data['tags']) && !empty($data['tags'])) {
      if ($where) {
        $query .= " AND t.tag_name IN (";
      } else {
        $query .= " WHERE t.tag_name IN (";
        $where = true;
      }
  
      // Menggunakan placeholder untuk setiap tag
      $tags = explode(',', $data['tags']);
      foreach ($tags as $index => $tag) {
        $query .= ":tag" . $index . ",";
        $binds['tag' . $index] = trim($tag);
      }
  
      // Menghapus koma ekstra dan menutup kurung
      $query = rtrim($query, ',') . ")";
    }
  
    $query .= " ORDER BY n.note_id";
    if (isset($data['sort'])) {
      if ($data['sort'] == 1) {
        $query .= " ASC";
      }
    } else {
      $query .= " DESC";
    }
  
    $query .= " LIMIT 4";
  
    // Menambahkan OFFSET jika halaman diberikan
    if (isset($data['page'])) {
      $query .= " OFFSET :offset";
      $binds['offset'] = $data['limit'] * ($data['page'] - 1);
    } else {
      $query .= " OFFSET 0";
    }
  
    // Persiapan prepared statement
    $this->db->query($query);
  
    // Bind parameters
    foreach ($binds as $key => $value) {
      $this->db->bind($key, $value);
    }
  
    // Eksekusi prepared statement
    $this->db->execute();
  
    return $this->db->fetchAll();
  }
  
  public function getTagsByNoteId($id)
  {
    $this->db->query('SELECT * FROM tag_notes WHERE note_id = :id');
    $this->db->bind('id', $id);
    return $this->db->fetchAll();
  }

  public function insertTags($note_id, $tags) {
    // Kueri SQL dengan prepared statement
    $sql = 'INSERT INTO tag_notes (note_id, tag_name) VALUES ';

    // Membuat array placeholder untuk setiap pasangan note_id dan tag_name
    $placeholders = array();
    $binds = array();

    foreach ($tags as $index => $tag) {
      $placeholders[] = '(:note_id, :tag_name' . $index . ')';
      $binds['tag_name' . $index] = $tag;
    }

    // Gabungkan semua placeholder dengan koma untuk membuat satu baris VALUES
    $sql .= implode(', ', $placeholders);
    
    $this->db->query($sql);

    // Bind parameter dengan nilai yang sesuai
    $this->db->bind('note_id', $note_id);
    return ($sql);

    foreach ($binds as $key => $value) {
      $this->db->bind($key, $value, PDO::PARAM_STR);
    }

    // Eksekusi prepared statement
    $this->db->execute();

    // Mengembalikan jumlah baris yang terpengaruh (jumlah tag yang berhasil disisipkan)
    return count($tags);
  }

  public function insertTag($note_id, $tags) {
    // Kueri SQL dengan prepared statement
    $sql = 'INSERT INTO tag_notes (note_id, tag_name) VALUES (:note_id, :tag_name)';
    $this->db->query($sql);
    // Bind parameter dengan nilai yang sesuai
    $this->db->bind('note_id', $note_id);
    $this->db->bind('tag_name', $tags, PDO::PARAM_STR);
    
    // Eksekusi prepared statement
    $this->db->execute();

    return true;
  }

  public function deleteTagsByNoteId($noteId) {
    // Kueri SQL dengan prepared statement untuk menghapus tag berdasarkan note_id
    $this->db->query('DELETE FROM tag_notes WHERE note_id = :noteId');

    // Bind parameter dengan nilai yang sesuai
    $this->db->bind(':noteId', $noteId, PDO::PARAM_INT);

    // Eksekusi prepared statement
    $this->db->execute();

    // Mengembalikan jumlah baris yang terpengaruh (jumlah tag yang dihapus)
    return $this->db->rowCount();
  }
}
