<?php
require_once __DIR__ .'/../../config/config.php';
require_once __DIR__ .'/../../app/core/Database.php';
require_once __DIR__ .'/../../app/core/DataSeeder.php';
class UserModel {
  private $table = 'users';
  private $db;

  public function __construct()
  {
    $this->db = new Database;

    $seedingStatusFile = __DIR__ . '/seeding.lock';

    if (!file_exists($seedingStatusFile)) {
      if (touch($seedingStatusFile)) {
          $dataSeeder = new DataSeeder();
          $dataSeeder->seedDatas();
      } else {}
    } else {}
  }

  public function getAllUser(){
    $this->db->query('SELECT * FROM users');
    return $this->db->fetchAll();
  }

  public function getUserTableInfos(){
    $this->db->query('SELECT user_id, username, email FROM users ORDER BY user_id');
    return $this->db->fetchAll();
  }

  public function getUserById($id)
  {
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE user_id = :id');
    $this->db->bind('id', $id);
    return $this->db->fetch();
  }
  
  public function getUserByEmail($email)
  {
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE email = :email');
    $this->db->bind('email', $email);
    return $this->db->fetch();
  }

  public function getUserByUsername($username)
  {
    $this->db->query('SELECT * FROM ' . $this->table . ' WHERE username = :username');
    $this->db->bind('username', $username);
    return $this->db->fetch();
  }

  public function register($data)
  {
    $this->db->query('INSERT INTO ' . $this->table . ' (username, email, password, is_admin) VALUES (:username, :email, :password, false)');
    $this->db->bind('username', $data['username']);
    $this->db->bind('email', $data['email']);
    $this->db->bind('password', password_hash($data['password'], PASSWORD_DEFAULT));
    $this->db->execute();
    return $this->db->rowCount();
  }

  public function deleteUserById($userId) {
    // Kueri SQL dengan prepared statement untuk menghapus tag berdasarkan note_id
    $this->db->query('DELETE FROM ' . $this->table . ' WHERE user_id = :userId');

    // Bind parameter dengan nilai yang sesuai
    $this->db->bind(':userId', $userId, PDO::PARAM_INT);

    // Eksekusi prepared statement
    $this->db->execute();

    // Mengembalikan jumlah baris yang terpengaruh (jumlah tag yang dihapus)
    return $this->db->rowCount();
  }

  public function updateUser($data) {
    // Kueri SQL dengan prepared statement untuk mengupdate user berdasarkan user_id
    $query = 'UPDATE ' . $this->table . ' SET email = :email, password = :password WHERE user_id = :user_id';
    $this->db->query($query);

    // Bind parameter dengan nilai yang sesuai
    $this->db->bind(':user_id', $data['user_id'], PDO::PARAM_INT);
    $this->db->bind(':email', $data['email'], PDO::PARAM_STR);
    $this->db->bind(':password', password_hash($data['password'], PASSWORD_DEFAULT), PDO::PARAM_STR);

    // Eksekusi prepared statement
    $this->db->execute();

    // Mengembalikan jumlah baris yang terpengaruh (jumlah user yang diupdate)
    return $this->db->rowCount();
  }
}
