let tag = getQueryVariable("tag") || ""
let sort = getQueryVariable("sort") || ""
let title = getQueryVariable("title") || ""
let page = parseInt(getQueryVariable("page")) || 1;
let searchTimeout; // Variabel untuk menyimpan timeout

function debounceSearchNote(userId, isAdmin) {
  clearTimeout(searchTimeout);
  searchTimeout = setTimeout(function () {
    searchNote("", userId, isAdmin);
  }, 500); 
}

function addNote(userId, isAdmin) {
  if (isAdmin) {
    window.location.href = "/?admin/addnote/" + userId;
  } else {
    window.location.href = "/?notes/addnote/";
  }
};

function openNote(noteId, isAdmin) {
  if (isAdmin) {
    window.location.href = "/?admin/notecontent/" + noteId;
  } else {
    window.location.href = "/?notes/content/" + noteId;
  }
}

function getQueryVariable(variable) {
  const query = window.location.search.substring(1);
  const vars = query.split('&');
  for (let i = 0; i < vars.length; i++) {
    const pair = vars[i].split('=');
    if (decodeURIComponent(pair[0]) === variable) {
      return decodeURIComponent(pair[1]);
    }
  }
  return "";
}

function updateQueryStringParameter(uri, key, value) {
  var re = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
  var separator = uri.indexOf("?") !== -1 ? "&" : "?";
  if (uri.match(re)) {
    return uri.replace(re, "$1" + key + "=" + value + "$2");
  } else {
    return uri + separator + key + "=" + value;
  }
}

const searchNote = (data = "", userId, isAdmin) => {
  let base_url = "../../app/api/notes/search.php?";
  let add_url = "";
  query_data = data

  try {
    if (document.getElementById("search-title-input")) {
      title = document.getElementById("search-title-input").value;
    } 

    if (title.length > 0) {
      add_url += "&title=" + title;
    }
    if (tag.length > 0) {
      add_url += "&tags=" + tag;
    }
    if (sort) {
      add_url += "&sort=" + sort;
    }

    if (page.length > 0) {
      add_url += "&page=" + page;
    } else {
      add_url += "&page=1";
    }    
    listPages(userId, isAdmin)
  } catch (e) {
    console.log(e);
  }

  const xhr = new XMLHttpRequest();

  if (data != "") {
    final_url = base_url + data;
    add_url = data;
  } else {
    final_url = base_url + add_url;
  }

  final_url += "&user_id=" + userId;

  xhr.open("GET", final_url, true);

  xhr.onload = function () {
    if (this.status == 200) {
      let response = JSON.parse(this.responseText);

      let notesCardsCont = document.getElementById("notes-cards-container");
      notesCardsCont.innerHTML = "";
      if (isAdmin) {
        window.history.pushState("", "", "/?admin/searchnotes/" +  userId + "/" + add_url);
      } else {
        window.history.pushState("", "", "/?notes/" + add_url);
      }
      
      if (response["status"] == "error") {
        notesCardsCont.innerHTML = `
        <div class="bino-sed-container" id="bino-sed-container">
          <p>Sorry, no notes found :(</p>
        </div>`;
      } else {
        response.forEach((note) => {
          notesCardsCont.innerHTML += `
              <div class="notes-card" onclick="openNote(${note.note_id}, ${isAdmin})">
                  <div class="note-thumbnail">
                      <img src="${note.image_path}" alt="note thumbnail" class="note-thumbnail-img">
                  </div>
                  <div class="notes-information">
                      <div class="title">
                          ${note.title}
                      </div>
                      <hr class="horizontal-line">

                      <div class="tags">
                        <div class="tag">
                          ${note.tag_name}                   
                        </div>
                      </div>
                  </div>       
              </div>
          `;
        });
      }
    } else {
      if (isAdmin) {
        window.location.href = "/?admin/searchnotes/" +  userId + "/&page=1" ;
      } else {
        window.location.href = "/?notes/&page=1" ;
      }
      alert("failed to search notes");
    }
  };
  xhr.send();
};

function toggleSort() {
  // Mengubah tampilan container
  let sortButton = document.getElementById('sort-button');
  if (sort == 1) {
    sort = "";
    sortButton.innerHTML = 
    `Date 
    <i class="fa fa-sort-amount-down"></i> 
    `;
  } else {
    sort = 1;
    sortButton.innerHTML = 
    `Date 
    <i class="fa fa-sort-amount-up"></i>`;
  }
}

function toggleChooseTag() {
  // Mengubah tampilan container
  let tagsListContainer = document.querySelector('.tags-list-container');
  if (tagsListContainer.style.display == 'none') {
    tagsListContainer.style.display = 'flex';
  } else {
    tagsListContainer.style.display = 'none';
  }
}

function chooseTag(chosenTag) {
  let tagsButton = document.querySelector('.tags-button');

  if (chosenTag == "All") {
    tag = ""
  } else {
    tag = chosenTag
  }
  tagsButton.textContent = chosenTag; 

  toggleChooseTag();
}

function setTitleInput() {
  // Dapatkan nilai parameter 'title' dari query string
  const queryString = window.location.search;
  const urlParams = new URLSearchParams(queryString);
  const titleParam = urlParams.get('title');

  // Temukan elemen input 'search-title-input'
  const searchTitleInput = document.getElementById('search-title-input');

  // Periksa apakah 'titleParam' ada dan isi input 'search-title-input' jika ada
  if (titleParam) {
    searchTitleInput.value = titleParam;
  }
}

function setPageNum() {
  const queryString = window.location.search;
  const urlParams = new URLSearchParams(queryString);
  // Dapatkan nilai parameter 'page' dari query string
  const pageParam = urlParams.get('page');

  // Temukan elemen select 'page-dropdown'
  const pageDropdown = document.getElementById('page-dropdown');

  // Periksa apakah 'pageParam' ada dan pilih opsi yang sesuai pada dropdown
  if (pageParam) {
    // Ubah nilai 'page-dropdown' ke 'pageParam'
    pageDropdown.value = pageParam;
  }
}

function listPages(userId=2, isAdmin) {
  try {
    // Parse query_data menjadi objek URLSearchParams
    const queryParams = new URLSearchParams();

    if (isAdmin) {
      if (userId) {
        queryParams.set('user_id', userId);
      }
    }
    if (title.length > 0) {
      queryParams.set('title', title);
    }
    if (tag.length > 0) {
      queryParams.set('tag', tag);
    }
    if (page.length > 0) {
      queryParams.set('tag', tag);
    }

    // Dapatkan string URL yang telah diperbarui
    const updatedQuery = queryParams.toString();

    const xhr = new XMLHttpRequest();
    xhr.open('GET', '../../app/api/notes/page.php?' + updatedQuery, true);

    xhr.onload = function () {
      if (xhr.readyState === 4 && xhr.status === 200) {
        const countNote = JSON.parse(xhr.responseText);
        const totalPages = Math.ceil(countNote.num_note / 4);
        
        let currentPage = parseInt(getQueryVariable("page")) || 1; 

        // Dapatkan elemen dropdown
        const pageDropdown = document.getElementById('page-dropdown');

        // Bersihkan pilihan yang ada jika ada
        pageDropdown.innerHTML = '';

        // Buat pilihan untuk setiap halaman
        for (let i = 1; i <= totalPages; i++) {
          const option = document.createElement('option');
          option.value = i;
          option.textContent = i;

          // Tandai pilihan yang sesuai dengan halaman saat ini
          if (i === currentPage) {
            option.selected = true;
          }

          pageDropdown.appendChild(option);
        }
      }
      setPageNum();
    };
    xhr.send();
  } catch (e) {
    console.log(e);
  }
}

// function changePage() {
function changePage(userId=2, isAdmin) {
  // Dapatkan nilai halaman yang dipilih dari dropdown
  const selectedPage = document.getElementById('page-dropdown').value;

  try {
    // Parse query_data menjadi objek URLSearchParams
    const queryParams = new URLSearchParams();

    // Setel nilai parameter 'page' menjadi selectedPage
    queryParams.set('page', selectedPage);
    if (title.length > 0) {
      queryParams.set('title', title);
    }
    if (tag.length > 0) {
      queryParams.set('tag', tag);
    }
    if (sort) {
      queryParams.set('sort', sort);
    }
    
    // Dapatkan string URL yang telah diperbarui
    const updatedQuery = queryParams.toString();

    if (isAdmin) {
      window.location.href = "/?admin/searchnotes/" + userId + "/&" + updatedQuery;
    } else {
      window.location.href = "/?notes/&" + updatedQuery;
    }
  } catch (e) {
    console.log(e);
  }
}
